<?php
/**
 * This is the entrypoint for the locrMAPS for WooCommerce WordPress plugin.
 *
 * @package locrMAPS for WooCommerce
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMapsForWooCommerce\Frontend;

use Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\ValueObjects\Size;
use Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\Repositories\Styles as StylesRepository;
use Locr\WordPress\Plugin\LocrMapsForWooCommerce\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the shortcode.
 */
final class Shortcode {
	/**
	 * This function initializes the shortcode.
	 */
	public static function init(): void {
		add_shortcode( 'locrmaps_icons', array( self::class, 'render_icons_shortcode' ) );
		add_shortcode( 'locrmaps_overlay_templates', array( self::class, 'render_overlay_templates_shortcode' ) );
		add_shortcode( 'locrmaps_overlay_templates_base_colors', array( self::class, 'render_overlay_templates_base_colors_shortcode' ) );
		add_shortcode( 'locrmaps_overlay_templates_text_colors', array( self::class, 'render_overlay_templates_text_colors_shortcode' ) );
		add_shortcode( 'locrmaps_sizes', array( self::class, 'render_sizes_shortcode' ) );
		add_shortcode( 'locrmaps_styles', array( self::class, 'render_styles_shortcode' ) );
		add_shortcode( 'locrmaps_text_overlay', array( self::class, 'render_text_overlay_shortcode' ) );
	}

	/**
	 * This function renders the icons-shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_icons_shortcode( array $atts = array() ): string {
		$options = get_option( 'locrmaps_for_woocommerce_options' );

		$rendered = '<select id="locrmaps-icons" name="locrmaps-icons">' . PHP_EOL;

		$icons = array();
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_icons'] ) && is_string( $options['locrmaps_for_woocommerce_icons'] ) ) {
			$icons_string = trim( $options['locrmaps_for_woocommerce_icons'] );
			if ( ! empty( $icons_string ) ) {
				$icons_exploded = explode( '|', $icons_string );
				foreach ( $icons_exploded as $icon ) {
					$trimmed_icon = trim( $icon );
					if ( empty( $trimmed_icon ) ) {
						continue;
					}
					$icons[] = $trimmed_icon;
				}
			}
		}

		if ( empty( $icons ) ) {
			$applied_filters_for_icons_names = apply_filters( 'locrmaps_icon_names', '' );
			if ( is_array( $applied_filters_for_icons_names ) ) {
				$icons = $applied_filters_for_icons_names;
			}
		}

		$icon_data = apply_filters( 'locrmaps_icon_data', '' );
		foreach ( $icons as $icon_name ) {
			if ( ! is_string( $icon_name ) ) {
				continue;
			}
			$rendered .= '<option value="' . esc_attr( $icon_name ) . '"';
			if (
				is_array( $icon_data ) &&
				isset( $icon_data[ $icon_name ] ) &&
				is_array( $icon_data[ $icon_name ] ) &&
				isset( $icon_data[ $icon_name ]['url'] ) &&
				is_string( $icon_data[ $icon_name ]['url'] )
			) {
				$rendered .= ' data-image-url="' . esc_attr( $icon_data[ $icon_name ]['url'] ) . '"';
			}
			$rendered .= '>' . esc_html( $icon_name ) . '</option>' . PHP_EOL;
		}

		$rendered .= '</select>' . PHP_EOL;

		$styling = '';
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout' ] ) ) {
			$styling = trim( $options[ Plugin::SLUG . '_selection_layout' ] );
		}
		if ( isset( $atts['styling'] ) && is_string( $atts['styling'] ) ) {
			$styling = trim( $atts['styling'] );
		}

		if ( 'wc-product-property' === $styling ) {
			$rendered = self::wrap_shortcode_for_wc_product_property_styling(
				shortcode: $rendered,
				id: 'locrmaps-icons',
				default_label: esc_html__( 'Icon', 'locrmaps-for-woocommerce-textdomain' )
			);
		}

		return $rendered;
	}

	/**
	 * This function renders the overlay templates-shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_overlay_templates_shortcode( array $atts = array() ): string {
		$options = get_option( 'locrmaps_for_woocommerce_options' );

		$rendered = '<select id="locrmaps-overlay-templates" name="locrmaps-overlay-templates">' . PHP_EOL;

		$overlay_templates = array();
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_overlay_templates'] ) && is_string( $options['locrmaps_for_woocommerce_overlay_templates'] ) ) {
			$overlay_templates_string = trim( $options['locrmaps_for_woocommerce_overlay_templates'] );
			if ( ! empty( $overlay_templates_string ) ) {
				$overlay_templates_exploded = explode( '|', $overlay_templates_string );
				foreach ( $overlay_templates_exploded as $overlay_template ) {
					$trimmed_overlay_template = trim( $overlay_template );
					if ( empty( $trimmed_overlay_template ) ) {
						continue;
					}
					$overlay_templates[] = $trimmed_overlay_template;
				}
			}
		}

		if ( empty( $overlay_templates ) ) {
			$overlay_templates = array( 'clean', 'heart', 'circle', 'pin' );
		}

		foreach ( $overlay_templates as $overlay_template ) {
			$relative_preview_path = 'public/assets/images/overlay-templates/preview_' . $overlay_template . '.png';
			$preview_image_file    = plugin_dir_path( dirname( __DIR__ ) ) . $relative_preview_path;
			$preview_image_url     = plugins_url( $relative_preview_path, dirname( __DIR__ ) );

			$rendered .= '<option value="' . esc_attr( $overlay_template ) . '"';
			if ( file_exists( $preview_image_file ) ) {
				$rendered .= ' data-image-url="' . esc_attr( $preview_image_url ) . '"';
			}
			$rendered .= '>' . esc_html( $overlay_template ) . '</option>' . PHP_EOL;
		}

		$rendered .= '</select>' . PHP_EOL;

		$styling = '';
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout' ] ) ) {
			$styling = trim( $options[ Plugin::SLUG . '_selection_layout' ] );
		}
		if ( isset( $atts['styling'] ) && is_string( $atts['styling'] ) ) {
			$styling = trim( $atts['styling'] );
		}
		if ( 'wc-product-property' === $styling ) {
			$rendered = self::wrap_shortcode_for_wc_product_property_styling(
				shortcode: $rendered,
				id: 'locrmaps-overlay-templates',
				default_label: esc_html__( 'Overlay Template', 'locrmaps-for-woocommerce-textdomain' )
			);
		}

		return $rendered;
	}

	/**
	 * This function renders the overlay templates base colors shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_overlay_templates_base_colors_shortcode( array $atts = array() ): string {
		$options = get_option( 'locrmaps_for_woocommerce_options' );

		$base_colors          = array();
		$base_colors['white'] = array(
			'name' => 'white',
		);
		$label_is_active      = false;

		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_overlay_templates_base_colors'] ) && is_string( $options['locrmaps_for_woocommerce_overlay_templates_base_colors'] ) ) {
			$base_colors_string = trim( $options['locrmaps_for_woocommerce_overlay_templates_base_colors'] );
			if ( ! empty( $base_colors_string ) ) {
				$base_colors_exploded = explode( '|', $base_colors_string );
				foreach ( $base_colors_exploded as $base_color_value ) {
					$trimmed_base_color = trim( $base_color_value );
					if ( empty( $trimmed_base_color ) ) {
						continue;
					}

					$color_value          = $trimmed_base_color;
					$base_color_meta_data = array(
						'name' => $color_value,
					);

					$pattern = get_shortcode_regex( array( 'color' ) );
					if ( preg_match( "/{$pattern}/", $trimmed_base_color, $matches ) ) {
						if ( isset( $matches[3] ) ) {
							$shortcode_atts = shortcode_parse_atts( $matches[3] );
							if ( isset( $shortcode_atts['value'] ) && is_string( $shortcode_atts['value'] ) && ! empty( trim( $shortcode_atts['value'] ) ) ) {
								$color_value                  = trim( $shortcode_atts['value'] );
								$base_color_meta_data['name'] = $color_value;
							}
							if ( isset( $shortcode_atts['name'] ) && is_string( $shortcode_atts['name'] ) ) {
								$base_color_meta_data['name'] = $shortcode_atts['name'];
								$label_is_active              = true;
							}
						}
					}
					$base_colors[ $color_value ] = $base_color_meta_data;
				}
			}
		}

		$rendered = '<select id="locrmaps-overlay-templates-base-colors" name="locrmaps-overlay-templates-base-colors">' . PHP_EOL;
		foreach ( $base_colors as $base_color_value => $base_color_meta_data ) {
			$rendered .= '<option value="' . esc_attr( $base_color_value ) . '"';
			$rendered .= ' data-use-value-as-preview-image-color="true"';
			if ( $label_is_active ) {
				$rendered .= ' data-label-is-active="true"';
			}
			$rendered .= '>' . esc_html( $base_color_meta_data['name'] ) . '</option>' . PHP_EOL;
		}
		$rendered .= '</select>';

		$styling = '';
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout' ] ) ) {
			$styling = trim( $options[ Plugin::SLUG . '_selection_layout' ] );
		}
		if ( isset( $atts['styling'] ) && is_string( $atts['styling'] ) ) {
			$styling = trim( $atts['styling'] );
		}
		if ( 'wc-product-property' === $styling ) {
			$rendered = self::wrap_shortcode_for_wc_product_property_styling(
				shortcode: $rendered,
				id: 'locrmaps-overlay-templates-base-colors',
				default_label: esc_html__( 'Overlay Template Base Colors', 'locrmaps-for-woocommerce-textdomain' )
			);
		}

		return $rendered;
	}

	/**
	 * This function renders the overlay templates text colors shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_overlay_templates_text_colors_shortcode( array $atts = array() ): string {
		$options = get_option( 'locrmaps_for_woocommerce_options' );

		$text_colors          = array();
		$text_colors['black'] = array(
			'name' => 'black',
		);
		$label_is_active      = false;

		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_overlay_templates_text_colors'] ) && is_string( $options['locrmaps_for_woocommerce_overlay_templates_text_colors'] ) ) {
			$text_colors_string = trim( $options['locrmaps_for_woocommerce_overlay_templates_text_colors'] );
			if ( ! empty( $text_colors_string ) ) {
				$text_colors_exploded = explode( '|', $text_colors_string );
				foreach ( $text_colors_exploded as $text_color_value ) {
					$trimmed_text_color = trim( $text_color_value );
					if ( empty( $trimmed_text_color ) ) {
						continue;
					}

					$color_value          = $trimmed_text_color;
					$text_color_meta_data = array(
						'name' => $color_value,
					);

					$pattern = get_shortcode_regex( array( 'color' ) );
					if ( preg_match( "/{$pattern}/", $trimmed_text_color, $matches ) ) {
						if ( isset( $matches[3] ) ) {
							$shortcode_atts = shortcode_parse_atts( $matches[3] );
							if ( isset( $shortcode_atts['value'] ) && is_string( $shortcode_atts['value'] ) && ! empty( trim( $shortcode_atts['value'] ) ) ) {
								$color_value                  = trim( $shortcode_atts['value'] );
								$text_color_meta_data['name'] = $color_value;
							}
							if ( isset( $shortcode_atts['name'] ) && is_string( $shortcode_atts['name'] ) ) {
								$text_color_meta_data['name'] = $shortcode_atts['name'];
								$label_is_active              = true;
							}
						}
					}
					$text_colors[ $color_value ] = $text_color_meta_data;
				}
			}
		}

		$rendered = '<select id="locrmaps-overlay-templates-text-colors" name="locrmaps-overlay-templates-text-colors">' . PHP_EOL;
		foreach ( $text_colors as $text_color_value => $text_color_meta_data ) {
			$rendered .= '<option value="' . esc_attr( $text_color_value ) . '"';
			$rendered .= ' data-use-value-as-preview-image-color="true"';
			if ( $label_is_active ) {
				$rendered .= ' data-label-is-active="true"';
			}
			$rendered .= '>' . esc_html( $text_color_meta_data['name'] ) . '</option>' . PHP_EOL;
		}
		$rendered .= '</select>';

		$styling = '';
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout' ] ) ) {
			$styling = trim( $options[ Plugin::SLUG . '_selection_layout' ] );
		}
		if ( isset( $atts['styling'] ) && is_string( $atts['styling'] ) ) {
			$styling = trim( $atts['styling'] );
		}
		if ( 'wc-product-property' === $styling ) {
			$rendered = self::wrap_shortcode_for_wc_product_property_styling(
				shortcode: $rendered,
				id: 'locrmaps-overlay-templates-text-colors',
				default_label: esc_html__( 'Overlay Template Text Colors', 'locrmaps-for-woocommerce-textdomain' )
			);
		}

		return $rendered;
	}

	/**
	 * This function renders the sizes-shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_sizes_shortcode( array $atts = array() ): string {
		$options = get_option( 'locrmaps_for_woocommerce_options' );

		$rendered = '<select id="locrmaps-sizes" name="locrmaps-sizes">' . PHP_EOL;

		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_sizes'] ) && is_string( $options['locrmaps_for_woocommerce_sizes'] ) ) {
			$sizes = trim( $options['locrmaps_for_woocommerce_sizes'] );
			if ( ! empty( $sizes ) ) {
				$shortcodes = explode( '|', $sizes );
				foreach ( $shortcodes as $index => $shortcode ) {
					try {
						$size      = Size::build_from_shortcode( $shortcode );
						$rendered .= '<option value="' . esc_attr( $size->build_option_value() ) . '">' . esc_html( $size->build_display_name() ) . '</option>' . PHP_EOL;
					} catch ( \Exception $e ) {
						if ( WP_DEBUG ) {
							// translators: %d: is the shortcode index.
							// translators: %s: is the error message.
							error_log( sprintf( __( 'The shortcode with the index [%1$d] is invalid. %2$s.', 'locrmaps-for-woocommerce-textdomain' ), $index, $e->getMessage() ) );
						}
					}
				}
			}
		}

		$rendered .= '</select>' . PHP_EOL;

		return $rendered;
	}

	/**
	 * This function renders the styles-shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_styles_shortcode( array $atts = array() ): string {
		$options = get_option( 'locrmaps_for_woocommerce_options' );

		$rendered = '<select id="locrmaps-styles" name="locrmaps-styles">' . PHP_EOL;

		$styles = array();
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_styles'] ) && is_string( $options['locrmaps_for_woocommerce_styles'] ) ) {
			$styles_string = trim( $options['locrmaps_for_woocommerce_styles'] );
			if ( ! empty( $styles_string ) ) {
				$styles_exploded = explode( '|', $styles_string );
				foreach ( $styles_exploded as $style ) {
					$trimmed_style = trim( $style );
					if ( empty( $trimmed_style ) ) {
						continue;
					}
					$styles[] = $trimmed_style;
				}
			}
		}

		if ( empty( $styles ) ) {
			$styles_repository = new StylesRepository();
			$styles            = $styles_repository->get_all();
		}

		foreach ( $styles as $style ) {
			$display_name                = $style;
			$overlay_template_base_color = '';
			$overlay_template_text_color = '';
			if ( ! is_numeric( $style ) ) {
				$pattern = get_shortcode_regex( array( 'style' ) );
				if ( preg_match( "/{$pattern}/", $style, $matches ) ) {
					if ( isset( $matches[3] ) ) {
						$shortcode_atts = shortcode_parse_atts( $matches[3] );
						if ( isset( $shortcode_atts['name'] ) && is_string( $shortcode_atts['name'] ) ) {
							$style = $shortcode_atts['name'];
						}
						if ( isset( $shortcode_atts['display-name'] ) && is_string( $shortcode_atts['display-name'] ) ) {
							$display_name = $shortcode_atts['display-name'];
						} else {
							$display_name = $style;
						}
						if ( isset( $shortcode_atts['overlay-template-base-color'] ) && is_string( $shortcode_atts['overlay-template-base-color'] ) ) {
							$overlay_template_base_color = $shortcode_atts['overlay-template-base-color'];
						}
						if ( isset( $shortcode_atts['overlay-template-text-color'] ) && is_string( $shortcode_atts['overlay-template-text-color'] ) ) {
							$overlay_template_text_color = $shortcode_atts['overlay-template-text-color'];
						}
					}
				}
			}
			$relative_preview_path = 'public/assets/images/style-templates/preview_' . $style . '.jpg';
			$preview_image_file    = plugin_dir_path( dirname( __DIR__ ) ) . $relative_preview_path;
			if ( ! file_exists( $preview_image_file ) ) {
				$relative_preview_path = 'public/assets/images/style-templates/preview_' . $style . '.jpeg';
				$preview_image_file    = plugin_dir_path( dirname( __DIR__ ) ) . $relative_preview_path;
			}
			$preview_image_url = plugins_url( $relative_preview_path, dirname( __DIR__ ) );

			$rendered .= '<option value="' . esc_attr( $style ) . '"';
			if ( file_exists( $preview_image_file ) ) {
				$rendered .= ' data-image-url="' . esc_attr( $preview_image_url ) . '"';
			}
			if ( ! empty( $overlay_template_base_color ) ) {
				$rendered .= ' data-overlay-template-base-color="' . esc_attr( $overlay_template_base_color ) . '"';
			}
			if ( ! empty( $overlay_template_text_color ) ) {
				$rendered .= ' data-overlay-template-text-color="' . esc_attr( $overlay_template_text_color ) . '"';
			}
			$rendered .= '>' . esc_html( $display_name ) . '</option>' . PHP_EOL;
		}

		$rendered .= '</select>' . PHP_EOL;

		$styling = '';
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout' ] ) ) {
			$styling = trim( $options[ Plugin::SLUG . '_selection_layout' ] );
		}
		if ( isset( $atts['styling'] ) && is_string( $atts['styling'] ) ) {
			$styling = trim( $atts['styling'] );
		}
		if ( 'wc-product-property' === $styling ) {
			$rendered = self::wrap_shortcode_for_wc_product_property_styling(
				shortcode: $rendered,
				id: 'locrmaps-styles',
				default_label: esc_html__( 'Style', 'locrmaps-for-woocommerce-textdomain' )
			);
		}

		return $rendered;
	}

	/**
	 * This function renders the text-overlay-shortcode.
	 *
	 * @param array<mixed> $atts The attributes.
	 */
	public static function render_text_overlay_shortcode( array $atts = array() ): string {
		$options = get_option( 'locrmaps_for_woocommerce_options' );

		$id = 'locrmaps-text-overlay';
		if ( isset( $atts['id'] ) && is_string( $atts['id'] ) && ! empty( trim( $atts['id'] ) ) ) {
			$id .= '-' . trim( $atts['id'] );
		} else {
			$id .= '-' . uniqid();
		}

		$is_multiline = false;
		if ( isset( $atts['multiline'] ) && is_string( $atts['multiline'] ) && ! empty( trim( $atts['multiline'] ) ) ) {
			$multiline    = strtolower( trim( $atts['multiline'] ) );
			$is_multiline = match ( $multiline ) {
				'true', '1', 'yes', 'on' => true,
				default => false,
			};
		}

		$rendered  = '<' . ( $is_multiline ? 'textarea rows="3"' : 'input type="text"' );
		$rendered .= ' id="' . esc_attr( $id ) . '" name="locrmaps-text-overlay" class="input-text"';
		if ( isset( $atts['overlay-ref-id'] ) && is_string( $atts['overlay-ref-id'] ) ) {
			$rendered .= ' data-overlay-ref-id="' . esc_attr( $atts['overlay-ref-id'] ) . '"';
		}
		if ( isset( $atts['placeholder'] ) && is_string( $atts['placeholder'] ) ) {
			$rendered .= ' placeholder="' . esc_attr( $atts['placeholder'] ) . '"';
		}
		if ( isset( $atts['x'] ) && is_string( $atts['x'] ) ) {
			$rendered .= ' data-x="' . esc_attr( $atts['x'] ) . '"';
		}
		if ( isset( $atts['y'] ) && is_string( $atts['y'] ) ) {
			$rendered .= ' data-y="' . esc_attr( $atts['y'] ) . '"';
		}
		if ( $is_multiline ) {
			$rendered .= '>';
			if ( isset( $atts['value'] ) && is_string( $atts['value'] ) ) {
				$rendered .= esc_html( $atts['value'] );
			}
			$rendered .= '</textarea>';
		} else {
			if ( isset( $atts['value'] ) && is_string( $atts['value'] ) ) {
				$rendered .= ' value="' . esc_attr( $atts['value'] ) . '"';
			}
			$rendered .= '/>';
		}
		$rendered .= PHP_EOL;

		$styling = '';
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout' ] ) ) {
			$styling = trim( $options[ Plugin::SLUG . '_selection_layout' ] );
		}
		if ( isset( $atts['styling'] ) && is_string( $atts['styling'] ) ) {
			$styling = trim( $atts['styling'] );
		}
		if ( 'wc-product-property' === $styling ) {
			$rendered = self::wrap_shortcode_for_wc_product_property_styling(
				shortcode: $rendered,
				id: $id,
				default_label: esc_html__( 'Text', 'locrmaps-for-woocommerce-textdomain' ),
				atts: $atts
			);
		}

		return $rendered;
	}

	/**
	 * This function wraps the shortcode for the WooCommerce product property styling.
	 *
	 * @param string       $shortcode     The shortcode.
	 * @param string       $id            The id.
	 * @param string       $default_label The label.
	 * @param array<mixed> $atts          The attributes.
	 */
	private static function wrap_shortcode_for_wc_product_property_styling( string $shortcode, string $id, string $default_label, array $atts = array() ): string {
		$label = $default_label;
		if ( isset( $atts['label'] ) && is_string( $atts['label'] ) ) {
			$label = trim( $atts['label'] );
		}
		$options = get_option( Plugin::SLUG . '_options' );
		if ( is_array( $options ) && ! isset( $options[ Plugin::SLUG . '_selection_layout_label_enabled' ] ) ) {
			$label = '';
		}

		$wrapped  = '<form action="javascript:void()" class="cart" style="margin-bottom: 1em;">';
		$wrapped .= '<table class="locrmaps-wc-styling wc-product-property variations" cellspacing="0" role="presentation" style="margin-bottom: initial;">';
		$wrapped .= '<tbody>';
		$wrapped .= '<tr style="margin-bottom: initial;">';
		$wrapped .= '<th class="label"';
		if ( empty( $label ) ) {
			$wrapped .= ' style="display: none;"';
		}
		$wrapped .= '>';
		$wrapped .= '<label for="' . esc_attr( $id ) . '">' . esc_html( $label ) . '</label>';
		$wrapped .= '</th>';
		$wrapped .= '<td class="value">';
		$wrapped .= $shortcode;
		$wrapped .= '</td>';
		$wrapped .= '</tr>';
		$wrapped .= '</tbody>';
		$wrapped .= '</table>';
		$wrapped .= '</form>';

		return $wrapped;
	}
}
